//$Id: UtilDAOImpl.java,v 1.10 2007/01/17 02:48:11 chaostone Exp $
/*
 *
 * Copyright c 2005-2009.
 * 
 * Licensed under GNU  LESSER General Public License, Version 3.  
 * http://www.gnu.org/licenses
 * 
 */
/********************************************************************************
 * @author pippo
 * 
 * MODIFICATION DESCRIPTION
 * 
 * Name                 Date                Description 
 * ============         ============        ============
 * pippo             2005-11-15         Created
 *  
 ********************************************************************************/

package org.beangle.model.persist.hibernate;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.Serializable;
import java.sql.Blob;
import java.sql.Clob;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.ListUtil;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.page.Page;
import org.beangle.commons.collection.page.PageLimit;
import org.beangle.commons.collection.page.SinglePage;
import org.beangle.model.Entity;
import org.beangle.model.entity.Model;
import org.beangle.model.persist.EntityDao;
import org.beangle.model.query.LimitQuery;
import org.beangle.model.query.QueryBuilder;
import org.beangle.model.query.builder.Condition;
import org.beangle.model.query.builder.OqlBuilder;
import org.hibernate.Criteria;
import org.hibernate.Hibernate;
import org.hibernate.Query;
import org.hibernate.SQLQuery;
import org.hibernate.Session;
import org.hibernate.criterion.Projection;
import org.hibernate.criterion.Projections;
import org.hibernate.engine.jdbc.StreamUtils;
import org.hibernate.impl.CriteriaImpl;
import org.hibernate.proxy.HibernateProxy;
import org.springframework.orm.hibernate3.support.HibernateDaoSupport;

/**
 * @author chaostone
 */
public class HibernateEntityDao extends HibernateDaoSupport implements EntityDao {

	@SuppressWarnings("unchecked")
	public <T> List<T> loadAll(Class<T> clazz) {
		String hql = "from " + Model.getEntityType(clazz).getEntityName();
		Query query = getSession().createQuery(hql);
		return query.list();
	}

	public <T> List<T> load(Class<T> entityClass, List<?> values) {
		return load(entityClass, "id", values.toArray());
	}

	public <T> List<T> load(Class<T> entityClass, Object[] values) {
		return load(entityClass, "id", values);
	}

	@SuppressWarnings("unchecked")
	public <T> List<T> load(Class<T> entityClass, String keyName, Object[] values) {
		if (entityClass == null || StringUtils.isEmpty(keyName) || values == null
				|| values.length == 0) {
			return Collections.emptyList();
		}
		String entityName = Model.getEntityType(entityClass).getEntityName();
		return (List<T>) load(entityName, keyName, values);
	}

	public List<?> load(String entityName, String keyName, Object[] values) {
		StringBuilder hql = new StringBuilder();
		hql.append("select entity from ").append(entityName).append(" as entity where entity.")
				.append(keyName).append(" in (:keyName)");
		Map<String, Object> parameterMap = MapUtil.newHashMap();
		if (values.length < 500) {
			parameterMap.put("keyName", values);
			return search(OqlBuilder.hql(hql.toString()).params(parameterMap).build());
		} else {
			QueryBuilder<?> query = OqlBuilder.hql(hql.toString()).params(parameterMap);
			List<Object> rs = ListUtil.newArrayList();
			int i = 0;
			while (i < values.length) {
				int end = i + 500;
				if (end > values.length) {
					end = values.length;
				}
				parameterMap.put("keyName", ArrayUtils.subarray(values, i, end));
				rs.addAll(search(query.build()));
				i += 500;
			}
			return rs;
		}
	}

	@SuppressWarnings("unchecked")
	public <T> T get(Class<T> clazz, Serializable id) {
		return (T) get(Model.getEntityType(clazz).getEntityName(), id);
	}

	@SuppressWarnings("unchecked")
	public Entity get(String entityName, Serializable id) {
		if (StringUtils.contains(entityName, '.')) {
			return (Entity) getHibernateTemplate().get(entityName, id);
		} else {
			String hql = "from " + entityName + " where id =:id";
			Query query = getSession().createQuery(hql);
			query.setParameter("id", id);
			List<Entity> rs = query.list();
			if (rs.isEmpty()) {
				return null;
			} else {
				return rs.get(0);
			}
		}
	}

	@SuppressWarnings("unchecked")
	public <T> T load(Class<T> clazz, Serializable id) {
		return (T) getHibernateTemplate().load(Model.getEntityType(clazz).getEntityName(), id);
	}

	/**
	 * 依据自构造的查询语句进行查询
	 * 
	 * @see #buildCountQueryStr(Query)
	 * @see org.beangle.entity.query.limit.Pagination
	 */
	@SuppressWarnings("unchecked")
	public List<?> search(org.beangle.model.query.Query query) {
		if (query instanceof LimitQuery) {
			org.beangle.model.query.LimitQuery<?> limitQuery = (org.beangle.model.query.LimitQuery) query;
			if (null == limitQuery.getLimit()) {
				return QuerySupport.find(limitQuery, getSession());
			} else {
				return new SinglePage(limitQuery.getLimit().getPageNo(), limitQuery.getLimit()
						.getPageSize(), QuerySupport.count(limitQuery, getSession()), QuerySupport
						.find(query, getSession()));
			}

		} else {
			return QuerySupport.find(query, getSession());
		}
	}

	/**
	 * 查询hql语句
	 * 
	 * @param <T>
	 * @param builder
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public <T> List<T> search(QueryBuilder<T> builder) {
		return (List<T>) search(builder.build());
	}

	public List<?> searchNamedQuery(final String queryName, final Map<String, Object> params) {
		Query query = this.getSession().getNamedQuery(queryName);
		return QuerySupport.setParameter(query, params).list();
	}

	public List<?> searchNamedQuery(final String queryName, final Map<String, Object> params,
			boolean cacheable) {
		Query query = this.getSession().getNamedQuery(queryName);
		query.setCacheable(cacheable);
		return QuerySupport.setParameter(query, params).list();
	}

	public List<?> searchNamedQuery(String queryName, Object[] params) {
		Query query = this.getSession().getNamedQuery(queryName);
		return QuerySupport.setParameter(query, params).list();
	}

	public List<?> searchHQLQuery(String hql) {
		return getSession().createQuery(hql).list();
	}

	public List<?> searchHQLQuery(String hql, Map<String, Object> params) {
		Query query = this.getSession().createQuery(hql);
		return QuerySupport.setParameter(query, params).list();
	}

	public List<?> searchHQLQuery(String hql, Object[] params) {
		Query query = this.getSession().createQuery(hql);
		return QuerySupport.setParameter(query, params).list();
	}

	public List<?> searchHQLQuery(String hql, final Map<String, Object> params, boolean cacheable) {
		Query query = this.getSession().createQuery(hql);
		query.setCacheable(cacheable);
		return QuerySupport.setParameter(query, params).list();
	}

	public Page<?> paginateNamedQuery(String queryName, Map<String, Object> params, PageLimit limit) {
		Query query = this.getSession().getNamedQuery(queryName);
		return paginateQuery(query, params, limit);
	}

	public Page<?> paginateHQLQuery(String hql, Map<String, Object> params, PageLimit limit) {
		Query query = this.getSession().createQuery(hql);
		return paginateQuery(query, params, limit);
	}

	@SuppressWarnings("unchecked")
	public Page<?> paginateCriteria(Criteria criteria, PageLimit limit) {
		CriteriaImpl criteriaImpl = (CriteriaImpl) criteria;
		int totalCount = 0;
		List<?> targetList = null;
		if (null == criteriaImpl.getProjection()) {
			criteria.setFirstResult((limit.getPageNo() - 1) * limit.getPageSize()).setMaxResults(
					limit.getPageSize());
			targetList = criteria.list();
			Projection projection = null;
			criteria.setFirstResult(0).setMaxResults(1);
			projection = Projections.rowCount();
			totalCount = ((Integer) criteria.setProjection(projection).uniqueResult()).intValue();
		} else {
			List<?> list = criteria.list();
			totalCount = list.size();
			criteria.setFirstResult((limit.getPageNo() - 1) * limit.getPageSize()).setMaxResults(
					limit.getPageSize());
			targetList = criteria.list();
		}
		// 返回结果
		return new SinglePage(limit.getPageNo(), limit.getPageSize(), totalCount, targetList);
	}

	public void evict(Object entity) {
		getSession().evict(entity);
	}

	public int executeUpdateHql(final String queryStr, final Object[] argument) {
		Query query = getSession().createQuery(queryStr);
		return QuerySupport.setParameter(query, argument).executeUpdate();
	}

	public int executeUpdateHql(final String queryStr, final Map<String, Object> parameterMap) {
		Query query = getSession().createQuery(queryStr);
		return QuerySupport.setParameter(query, parameterMap).executeUpdate();
	}

	public int executeUpdateNamedQuery(final String queryName,
			final Map<String, Object> parameterMap) {
		Query query = getSession().getNamedQuery(queryName);
		return QuerySupport.setParameter(query, parameterMap).executeUpdate();
	}

	public int executeUpdateNamedQuery(final String queryName, final Object[] argument) {
		Query query = getSession().getNamedQuery(queryName);
		return QuerySupport.setParameter(query, argument).executeUpdate();
	}

	public Blob createBlob(InputStream inputStream, int length) {
		return Hibernate.getLobCreator(getSession()).createBlob(inputStream, length);
	}

	public Blob createBlob(InputStream inputStream) {
		try {
			ByteArrayOutputStream buffer = new ByteArrayOutputStream(inputStream.available());
			StreamUtils.copy(inputStream, buffer);
			return Hibernate.getLobCreator(getSession()).createBlob(buffer.toByteArray());
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public Clob createClob(String str) {
		return Hibernate.getLobCreator(getSession()).createClob(str);
	}

	public void refresh(Object entity) {
		getSession().refresh(entity);
	}

	public void initialize(Object entity) {
		getHibernateTemplate().initialize(entity);
	}

	/**
	 * 
	 * @param query
	 * @param names
	 * @param values
	 * @param pageNo
	 * @param pageSize
	 * @return
	 */
	@SuppressWarnings("unchecked")
	public Page<?> paginateQuery(Query query, Map<String, Object> params, PageLimit limit) {
		QuerySupport.setParameter(query, params);
		query.setFirstResult((limit.getPageNo() - 1) * limit.getPageSize()).setMaxResults(
				limit.getPageSize());
		List<?> targetList = query.list();

		String queryStr = buildCountQueryStr(query);
		Query countQuery = null;
		if (query instanceof SQLQuery) {
			countQuery = getSession().createSQLQuery(queryStr);
		} else {
			countQuery = getSession().createQuery(queryStr);
		}
		QuerySupport.setParameter(countQuery, params);
		// 返回结果
		return new SinglePage(limit.getPageNo(), limit.getPageSize(), ((Number) (countQuery
				.uniqueResult())).intValue(), targetList);
	}

	public void saveOrUpdate(Object entity) {
		if (entity instanceof Collection<?>) {
			Collection<?> entities = (Collection<?>) entity;
			for (Iterator<?> iter = entities.iterator(); iter.hasNext();) {
				Object obj = iter.next();
				if (obj instanceof HibernateProxy) {
					getHibernateTemplate().saveOrUpdate(obj);
				} else {
					getHibernateTemplate().saveOrUpdate(
							Model.getEntityType(obj.getClass()).getEntityName(), obj);
				}
			}
		} else {
			if (entity instanceof HibernateProxy) {
				getHibernateTemplate().saveOrUpdate(entity);
			} else {
				getHibernateTemplate().saveOrUpdate(
						Model.getEntityType(entity.getClass()).getEntityName(), entity);
			}
		}
	}

	public void saveOrUpdate(String entityName, Object entity) {
		if (entity instanceof Collection<?>) {
			Collection<?> entities = (Collection<?>) entity;
			for (Iterator<?> iter = entities.iterator(); iter.hasNext();) {
				getHibernateTemplate().saveOrUpdate(entityName, (Object) iter.next());
			}
		} else {
			getHibernateTemplate().saveOrUpdate(entityName, entity);
		}
	}

	public void remove(Collection<?> entities) {
		for (Iterator<?> iter = entities.iterator(); iter.hasNext();) {
			getHibernateTemplate().delete((Object) iter.next());
		}
	}

	public void remove(Object entity) {
		getHibernateTemplate().delete(entity);
	}

	public boolean remove(Class<?> clazz, String attr, Object[] values) {
		if (clazz == null || StringUtils.isEmpty(attr) || values == null || values.length == 0) {
			return false;
		}
		String entityName = Model.getEntityType(clazz).getEntityName();
		StringBuilder hql = new StringBuilder();
		hql.append("delete from ").append(entityName).append(" where ").append(attr).append(
				" in (:ids)");
		Map<String, Object> parameterMap = MapUtil.newHashMap();
		parameterMap.put("ids", values);
		return executeUpdateHql(hql.toString(), parameterMap) > 0;
	}

	public boolean remove(Class<?> entityClass, String attr, Collection<?> values) {
		return remove(entityClass, attr, values.toArray());
	}

	public boolean remove(Class<?> clazz, Map<String, Object> keyMap) {
		if (clazz == null || keyMap == null || keyMap.isEmpty()) {
			return false;
		}
		String entityName = Model.getEntityType(clazz).getEntityName();
		StringBuilder hql = new StringBuilder();
		hql.append("delete from ").append(entityName).append(" where ");
		Set<String> keySet = keyMap.keySet();
		Map<String, Object> params = MapUtil.newHashMap();
		for (final String keyName : keySet) {
			Object keyValue = keyMap.get(keyName);
			String paramName = keyName.replace('.', '_');
			params.put(paramName, keyValue);
			if (keyValue.getClass().isArray() || keyValue instanceof Collection<?>) {
				hql.append(keyName).append(" in (:").append(paramName).append(") and ");
			} else {
				hql.append(keyName).append(" = :").append(paramName).append(" and ");
			}
		}
		hql.append(" (1=1) ");
		return (executeUpdateHql(hql.toString(), params) > 0);
	}

	/**
	 * 构造查询记录数目的查询字符串
	 * 
	 * @param query
	 * @return
	 */
	private String buildCountQueryStr(Query query) {
		String queryStr = "select count(*) ";
		if (query instanceof SQLQuery) {
			queryStr += "from (" + query.getQueryString() + ")";
		} else {
			String lowerCaseQueryStr = query.getQueryString().toLowerCase();
			String selectWhich = lowerCaseQueryStr.substring(0, query.getQueryString().indexOf(
					"from"));
			int indexOfDistinct = selectWhich.indexOf("distinct");
			int indexOfFrom = lowerCaseQueryStr.indexOf("from");
			// 如果含有distinct
			if (-1 != indexOfDistinct) {
				if (StringUtils.contains(selectWhich, ",")) {
					queryStr = "select count("
							+ query.getQueryString().substring(indexOfDistinct,
									query.getQueryString().indexOf(",")) + ")";

				} else {
					queryStr = "select count("
							+ query.getQueryString().substring(indexOfDistinct, indexOfFrom) + ")";
				}
			}
			queryStr += query.getQueryString().substring(indexOfFrom);
		}
		return queryStr;
	}

	public static final class QuerySupport {
		private QuerySupport() {
			super();
		}

		private static Query buildHibernateQuery(org.beangle.model.query.Query<?> bquery,
				final Session hibernateSession) {
			Query hibernateQuery = null;
			if (bquery.getLang().equals(org.beangle.model.query.Lang.HQL)) {
				hibernateQuery = hibernateSession.createQuery(bquery.getStatement());
			} else {
				hibernateQuery = hibernateSession.createSQLQuery(bquery.getStatement());
			}
			if (bquery.isCacheable()) {
				hibernateQuery.setCacheable(bquery.isCacheable());
			}
			setParameter(hibernateQuery, bquery.getParams());
			return hibernateQuery;
		}

		/**
		 * 统计该查询的记录数
		 * 
		 * @param query
		 * @param hibernateSession
		 * @return
		 */
		public static int count(final org.beangle.model.query.LimitQuery<?> limitQuery,
				final Session hibernateSession) {
			final org.beangle.model.query.Query<?> cntQuery = limitQuery.getCountQuery();
			if (null == cntQuery) {
				Query hibernateQuery = buildHibernateQuery(limitQuery, hibernateSession);
				return hibernateQuery.list().size();
			} else {
				Query hibernateQuery = buildHibernateQuery(cntQuery, hibernateSession);
				final Number count = (Number) (hibernateQuery.uniqueResult());
				if (null == count) {
					return 0;
				} else {
					return count.intValue();
				}
			}
		}

		/**
		 * 查询结果集
		 * 
		 * @param query
		 * @param hibernateSession
		 * @return
		 */
		public static List<?> find(final org.beangle.model.query.Query<?> query,
				final Session hibernateSession) {
			if (query instanceof LimitQuery<?>) {
				LimitQuery<?> limitQuery = (LimitQuery<?>) query;
				Query hibernateQuery = buildHibernateQuery(limitQuery, hibernateSession);
				if (null == limitQuery.getLimit()) {
					return hibernateQuery.list();
				} else {
					final PageLimit limit = limitQuery.getLimit();
					hibernateQuery.setFirstResult((limit.getPageNo() - 1) * limit.getPageSize())
							.setMaxResults(limit.getPageSize());
					return hibernateQuery.list();
				}
			} else {
				return buildHibernateQuery(query, hibernateSession).list();
			}
		}

		/**
		 * 为query设置参数
		 * 
		 * @param query
		 * @param argument
		 * @return
		 */
		public static Query setParameter(final Query query, final Object[] argument) {
			if (argument != null && argument.length > 0) {
				for (int i = 0; i < argument.length; i++) {
					query.setParameter(i, argument[i]);
				}
			}
			return query;
		}

		/**
		 * 为query设置参数
		 * 
		 * @param query
		 * @param argument
		 * @return
		 */
		public static Query setParameter(final Query query, final Map<String, Object> parameterMap) {
			if (parameterMap != null && !parameterMap.isEmpty()) {
				for (final Iterator<String> ite = parameterMap.keySet().iterator(); ite.hasNext();) {
					final String parameterName = ite.next();
					if (null == parameterName) {
						break;
					}
					final Object parameterValue = parameterMap.get(parameterName);
					if (null == parameterValue) {
						query.setParameter(parameterName, (Object) null);
					} else if (parameterValue.getClass().isArray()) {
						query.setParameterList(parameterName, (Object[]) parameterValue);
					} else if (parameterValue instanceof Collection<?>) {
						query.setParameterList(parameterName, (Collection<?>) parameterValue);
					} else {
						query.setParameter(parameterName, parameterValue);
					}
				}
			}
			return query;
		}
		/**
		 * 针对查询条件绑定查询的值
		 * 
		 * @param query
		 * @param conditions
		 */
		public static void bindValues(final Query query, final List<Condition> conditions) {
			int position = 0;
			boolean hasInterrogation = false; // 含有问号
			for (final Iterator<Condition> iter = conditions.iterator(); iter.hasNext();) {
				final Condition condition = (Condition) iter.next();
				if (StringUtils.contains(condition.getContent(), "?")) {
					hasInterrogation = true;
				}
				if (hasInterrogation) {
					for (final Iterator<?> iterator = condition.getParams().iterator(); iterator.hasNext();) {
						query.setParameter(position++, iterator.next());
					}
				} else {
					final List<String> paramNames = condition.getParamNames();
					for (int i = 0; i < paramNames.size(); i++) {
						final String name = paramNames.get(i);
						final Object value = condition.getParams().get(i);

						if (value.getClass().isArray()) {
							query.setParameterList(name, (Object[]) value);
						} else if (value instanceof Collection<?>) {
							query.setParameterList(name, (Collection<?>) value);
						} else {
							query.setParameter(name, value);
						}
					}
				}
			}
		}
	}
}