package org.beangle.model.persist.impl;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.beangle.commons.collection.MapUtil;
import org.beangle.commons.collection.page.Page;
import org.beangle.commons.collection.page.PageLimit;
import org.beangle.model.Entity;
import org.beangle.model.entity.Model;
import org.beangle.model.entity.ModelBuilder;
import org.beangle.model.persist.EntityDao;
import org.beangle.model.persist.EntityService;
import org.beangle.model.query.Query;
import org.beangle.model.query.QueryBuilder;

public class EntityServiceImpl implements EntityService {

	protected EntityDao entityDao;

	protected ModelBuilder modelBuilder;

	public <T> T load(Class<T> clazz, Serializable id) {
		return entityDao.load(clazz, id);
	}

	public <T> List<T> loadAll(Class<T> entity) {
		return entityDao.loadAll(entity);
	}

	public <T> T get(Class<T> clazz, Serializable id) {
		return entityDao.get(clazz, id);
	}

	public Entity<?> get(String entityName, Serializable id) {
		return entityDao.get(entityName, id);
	}

	public <T> List<T> load(Class<T> entity, String attr, Collection<?> values) {
		return entityDao.load(entity, attr, values.toArray());
	}

	public <T> List<T> load(Class<T> entity, String attr, Object value) {
		return entityDao.load(entity, attr, new Object[] { value });
	}

	public <T> List<T> load(Class<T> entity, String attr, Object[] values) {
		return entityDao.load(entity, attr, values);
	}

	public List<?> load(String entityName, String attr, Object[] values) {
		return entityDao.load(entityName, attr, values);
	}

	public <T> List<T> load(Class<T> entity, String[] attrs, Object[] values) {
		Map<String, Object> params = MapUtil.newHashMap();
		for (int i = 0; i < attrs.length; i++) {
			params.put(attrs[i], values[i]);
		}
		return load(entity, params);
	}

	/**
	 * @param entity
	 * @param parameterMap
	 * @return
	 */
	public <T> List<T> load(Class<T> entity, final Map<String, Object> parameterMap) {
		if (entity == null || parameterMap == null || parameterMap.isEmpty()) {
			return Collections.emptyList();
		}
		String entityName = entity.getName();
		StringBuilder hql = new StringBuilder();
		hql.append("select entity from ").append(entityName).append(" as entity ")
				.append(" where ");

		Map<String, Object> m = new HashMap<String, Object>(parameterMap.keySet().size());
		// 变量编号
		int i = 0;
		for (final String keyName : parameterMap.keySet()) {
			if (StringUtils.isEmpty(keyName)) {
				return null;
			}
			i++;
			Object keyValue = parameterMap.get(keyName);

			String[] tempName = StringUtils.split(keyName, "\\.");
			String name = tempName[tempName.length - 1] + i;
			m.put(name, keyValue);

			if (keyValue != null
					&& (keyValue.getClass().isArray() || keyValue instanceof Collection<?>)) {
				hql.append("entity.").append(keyName).append(" in (:").append(name)
						.append(") and ");
			} else {
				hql.append("entity.").append(keyName).append(" = :").append(name).append(" and ");
			}
		}
		hql.append(" (1=1) ");
		@SuppressWarnings("unchecked")
		List<T> rs = (List<T>) entityDao.searchHQLQuery(hql.toString(), m);
		return rs;
	}

	public List<?> search(Query<?> query) {
		return entityDao.search(query);
	}

	public <T> List<T> search(QueryBuilder<T> builder) {
		return entityDao.search(builder);
	}

	public Page<?> paginateHQLQuery(String hql, Map<String, Object> params, PageLimit limit) {
		return entityDao.paginateHQLQuery(hql, params, limit);
	}

	public Page<?> paginateNamedQuery(String queryName, Map<String, Object> params, PageLimit limit) {
		return entityDao.paginateNamedQuery(queryName, params, limit);
	}

	public List<?> searchHQLQuery(String hql, Map<String, Object> params, boolean cacheable) {
		return entityDao.searchHQLQuery(hql, params, cacheable);
	}

	public List<?> searchHQLQuery(String hql, Map<String, Object> params) {
		return entityDao.searchHQLQuery(hql, params);
	}

	public List<?> searchHQLQuery(String hql, Object[] params) {
		return entityDao.searchHQLQuery(hql, params);
	}

	public List<?> searchHQLQuery(String hql) {
		return entityDao.searchHQLQuery(hql);
	}

	public List<?> searchNamedQuery(String queryName, Map<String, Object> params, boolean cacheable) {
		return entityDao.searchNamedQuery(queryName, params, cacheable);
	}

	public List<?> searchNamedQuery(String queryName, Map<String, Object> params) {
		return entityDao.searchNamedQuery(queryName, params);
	}

	public List<?> searchNamedQuery(String queryName, Object[] params) {
		return entityDao.searchNamedQuery(queryName, params);
	}

	public int update(Class<?> entityClass, String attr, Object[] values, String[] argumentName,
			Object[] argumentValue) {
		if (null == values || values.length == 0) {
			return 0;
		}
		Map<String, Object> updateParams = MapUtil.newHashMap();
		for (int i = 0; i < argumentValue.length; i++) {
			updateParams.put(argumentName[i], argumentValue[i]);
		}
		return update(entityClass, attr, values, updateParams);

	}

	public int update(Class<?> entityClass, String attr, Object[] values,
			Map<String, Object> updateParams) {
		if (null == values || values.length == 0 || updateParams.isEmpty()) {
			return 0;
		}
		String entityName = entityClass.getName();
		StringBuilder hql = new StringBuilder();
		hql.append("update ").append(entityName).append(" set ");
		for (final String parameterName : updateParams.keySet()) {
			if (null == parameterName) {
				continue;
			}
			hql.append(parameterName).append(" = ").append(":").append(
					StringUtils.replaceChars(parameterName, '.', '_')).append(",");
		}
		hql.deleteCharAt(hql.length() - 1);
		hql.append(" where ").append(attr).append(" in (:ids)");
		updateParams.put("ids", values);
		return entityDao.executeUpdateHql(hql.toString(), updateParams);
	}

	public void remove(Object entity) {
		entityDao.remove(entity);
	}

	public void remove(Collection<?> entities) {
		if (null != entities && !entities.isEmpty()) {
			entityDao.remove(entities);
		}
	}

	public boolean remove(Class<?> entityClass, Map<String, Object> parameterMap) {
		return entityDao.remove(entityClass, parameterMap);
	}

	public boolean remove(Class<?> entityClass, String attr, Collection<?> values) {
		return entityDao.remove(entityClass, attr, values);
	}

	public boolean remove(Class<?> entityClass, String attr, Object[] values) {
		return entityDao.remove(entityClass, attr, values);
	}

	public void saveOrUpdate(Collection<?> entities) {
		if (null != entities && !entities.isEmpty()) {
			entityDao.saveOrUpdate(entities);
		}
	}

	public void saveOrUpdate(Object entity) {
		if (entity != null) {
			entityDao.saveOrUpdate(entity);
		}
	}

	public void saveOrUpdate(String entityName, Collection<?> entities) {
		entityDao.saveOrUpdate(entityName, entities);
	}

	public void saveOrUpdate(String entityName, Object entity) {
		entityDao.saveOrUpdate(entityName, entity);
	}

	public int count(String entityName, String keyName, Object value) {
		String hql = "select count(*) from " + entityName + " where " + keyName + "=:value";
		Map<String, Object> params = MapUtil.newHashMap();
		params.put("value", value);
		List<?> rs = entityDao.searchHQLQuery(hql, params);
		if (rs.isEmpty()) {
			return 0;
		} else {
			return ((Number) rs.get(0)).intValue();
		}
	}

	public int count(Class<?> entityClass, String keyName, Object value) {
		return count(entityClass.getName(), keyName, value);
	}

	public int count(Class<?> entityClass, String[] attrs, Object[] values, String countAttr) {
		String entityName = entityClass.getName();
		StringBuilder hql = new StringBuilder();
		if (StringUtils.isNotEmpty(countAttr)) {
			hql.append("select count(distinct ").append(countAttr).append(") from ");
		} else {
			hql.append("select count(*) from ");
		}
		hql.append(entityName).append(" as entity where ");
		Map<String, Object> params = MapUtil.newHashMap();
		for (int i = 0; i < attrs.length; i++) {
			String keyName = (String) attrs[i];
			if (StringUtils.isEmpty(keyName)) {
				continue;
			}
			Object keyValue = values[i];
			params.put(keyName, keyValue);
			String[] tempName = StringUtils.split(attrs[i], "\\.");
			attrs[i] = tempName[tempName.length - 1] + i;
			if (keyValue != null
					&& (keyValue.getClass().isArray() || keyValue instanceof Collection<?>)) {
				hql.append("entity.").append(keyName).append(" in (:").append(attrs[i]).append(
						") and ");
			} else {
				hql.append("entity.").append(keyName).append(" = :").append(attrs[i]).append(
						" and ");
			}
		}
		hql.append(" (1=1) ");
		return ((Number) entityDao.searchHQLQuery(hql.toString(), params).get(0)).intValue();
	}

	public boolean exist(Class<?> entityClass, String attr, Object value) {
		return count(entityClass, attr, value) > 0;
	}

	public boolean exist(String entityName, String attr, Object value) {
		return count(entityName, attr, value) > 0;
	}

	public boolean exist(Class<?> entity, String[] attrs, Object[] values) {
		return (count(entity, attrs, values, null) > 0);
	}

	/**
	 * 检查持久化对象是否存在
	 * 
	 * @param entityName
	 * @param keyName
	 * @param id
	 * @return boolean(是否存在) 如果entityId为空或者有不一样的entity存在则认为存在。
	 */
	public boolean duplicate(Class<? extends Entity<?>> clazz, Long id, String codeName,
			Object codeValue) {
		if (null != codeValue && StringUtils.isNotEmpty(codeValue.toString())) {
			List<? extends Entity<?>> list = entityDao.load(clazz, codeName,
					new Object[] { codeValue });
			if (list != null && !list.isEmpty()) {
				if (id == null) {
					return true;
				} else {
					for (Iterator<? extends Entity<?>> it = list.iterator(); it.hasNext();) {
						Entity<?> info = it.next();
						if (!info.getEntityId().equals(id)) {
							return true;
						}
					}
					return false;
				}
			}
		}
		return false;
	}

	public void initialize(Object entity) {
		entityDao.initialize(entity);
	}

	public void refresh(Object entity) {
		entityDao.refresh(entity);
	}

	public void evict(Object entity) {
		entityDao.evict(entity);
	}

	public void setEntityDao(EntityDao entityDao) {
		this.entityDao = entityDao;
	}

	public EntityDao getEntityDao() {
		return entityDao;
	}

	public boolean duplicate(String entityName, Long id, Map<String, Object> params) {
		StringBuilder b = new StringBuilder("from ");
		b.append(entityName).append(" where (1=1)");
		Map<String, Object> paramsMap = MapUtil.newHashMap();
		int i = 0;
		for (Iterator<String> iterator = params.keySet().iterator(); iterator.hasNext(); i++) {
			String key = iterator.next();
			b.append(" and ").append(key).append('=').append(":param" + i);
			paramsMap.put("param" + i, params.get(key));
		}
		List<?> list = entityDao.searchHQLQuery(b.toString(), paramsMap);
		if (!list.isEmpty()) {
			if (null == id) {
				return false;
			} else {
				for (Iterator<?> iter = list.iterator(); iter.hasNext();) {
					Entity<?> one = (Entity<?>) iter.next();
					if (!one.getEntityId().equals(id)) {
						return false;
					}
				}
			}
		}
		return true;
	}

	protected Model getModel() {
		return modelBuilder.getModel();
	}

	protected ModelBuilder getModelBuilder() {
		return modelBuilder;
	}

	public void setModelBuilder(ModelBuilder modelBuilder) {
		this.modelBuilder = modelBuilder;
	}

}
