package org.beangle.model.pojo;

import org.apache.commons.lang.builder.HashCodeBuilder;
import org.beangle.model.predicates.ValidEntityKeyPredicate;

public class LongIdObject implements LongIdEntity {
	private static final long serialVersionUID = -7530111699332363124L;

	protected Long id;

	public LongIdObject() {
		super();
	}

	public LongIdObject(Long id) {
		super();
		this.id = id;
	}

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getEntityId() {
		return id;
	}

	public boolean saved() {
		return ValidEntityKeyPredicate.INSTANCE.evaluate(id);
	}

	public boolean unsaved() {
		return !ValidEntityKeyPredicate.INSTANCE.evaluate(id);
	}

	/**
	 * @see java.lang.Object#hashCode()
	 */
	public int hashCode() {
		return new HashCodeBuilder(-64900959, -454788261).append(this.id).toHashCode();
	}

	/**
	 * 比较id,如果任一方id是null,则不相等
	 * <p>
	 * 因为业务对象被CGlib或者javassist增强的原因，这里只提供一般的基于id的比较,不提供基于Class的比较。如果在存在继承结构，
	 * 请重置equals方法。
	 * 
	 * @see java.lang.Object#equals(Object)
	 */
	public boolean equals(final Object object) {
		if (!(object instanceof LongIdObject)) {
			return false;
		}
		LongIdObject rhs = (LongIdObject) object;
		if (null == getId() || null == rhs.getId()) {
			return false;
		}
		return getId().equals(rhs.getId());
	}

	public boolean isPO() {
		return saved();
	}

	public boolean isVO() {
		return unsaved();
	}
	
}
