package org.beangle.model.transfer.io;

import java.util.Map;

import org.beangle.commons.collection.MapUtil;

public class MapedFormatRegistry implements TransferFormatRegistry {

	Map<String, Class<? extends Writer>> writers = MapUtil.newHashMap();

	Map<String, Class<? extends Reader>> readers = MapUtil.newHashMap();

	public Writer getWriter(String format) {
		Class<? extends Writer> writerClass = writers.get(format);
		if (null == writerClass) {
			return null;
		}
		try {
			return writerClass.newInstance();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public Reader getReader(String format) {
		Class<? extends Reader> readerClass = readers.get(format);
		if (null == readerClass) {
			return null;
		}
		try {
			return readerClass.newInstance();
		} catch (Exception e) {
			throw new RuntimeException(e);
		}
	}

	public <T extends Writer> void registerWriter(String format, Class<T> writerClazz) {
		writers.put(format, writerClazz);
	}

	public <T extends Reader> void registerReader(String format, Class<T> readerClazz) {
		readers.put(format, readerClazz);
	}

	public void unRegisterWriter(String format) {
		writers.remove(format);
	}

	public void unRegisterReader(String format) {
		readers.remove(format);
	}

}
